{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2020                                      }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.Calendar;

interface

uses
  {$IFNDEF LCLTMSWEB}
  Classes, WEBLib.Controls, VCL.Graphics, Types, Winapi.Windows;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, LCLType, LCLIntf, Graphics, Types, WEBLib.Controls;
  {$ENDIF}

type
  TCalendarInactiveDays = class(TPersistent)
  private
    FSunday: Boolean;
    FWednesday: Boolean;
    FTuesday: Boolean;
    FFriday: Boolean;
    FThursday: Boolean;
    FMonday: Boolean;
    FSaturday: Boolean;
    FOnChanged: TNotifyEvent;
    procedure SetFriday(const Value: Boolean);
    procedure SetMonday(const Value: Boolean);
    procedure SetSaturday(const Value: Boolean);
    procedure SetSunday(const Value: Boolean);
    procedure SetThursday(const Value: Boolean);
    procedure SetTuesday(const Value: Boolean);
    procedure SetWednesday(const Value: Boolean);
  protected
    procedure Changed;
  public
    constructor Create; virtual;
    function DateEnabledByIndex(AIndex: Integer): Boolean;
    property OnChanged: TNotifyEvent read FOnChanged write FOnChanged;
  published
    property Sunday: Boolean read FSunday write SetSunday default False;
    property Monday: Boolean read FMonday write SetMonday default False;
    property Tuesday: Boolean read FTuesday write SetTuesday default False;
    property Wednesday: Boolean read FWednesday write SetWednesday default False;
    property Thursday: Boolean read FThursday write SetThursday default False;
    property Friday: Boolean read FFriday write SetFriday default False;
    property Saturday: Boolean read FSaturday write SetSaturday default False;
  end;

  TCalendarWeekDayNames = class(TPersistent)
  private
    FSunday: string;
    FWednesday: string;
    FTuesday: string;
    FFriday: string;
    FThursday: string;
    FMonday: string;
    FSaturday: string;
    FOnChanged: TNotifyEvent;
    procedure SetFriday(const Value: string);
    procedure SetMonday(const Value: string);
    procedure SetSaturday(const Value: string);
    procedure SetSunday(const Value: string);
    procedure SetThursday(const Value: string);
    procedure SetTuesday(const Value: string);
    procedure SetWednesday(const Value: string);
  protected
    procedure Changed;
  public
    constructor Create; virtual;
    function DayNameByIndex(AIndex: Integer): string;
    property OnChanged: TNotifyEvent read FOnChanged write FOnChanged;
  published
    property Sunday: string read FSunday write SetSunday;
    property Monday: string read FMonday write SetMonday;
    property Tuesday: string read FTuesday write SetTuesday;
    property Wednesday: string read FWednesday write SetWednesday;
    property Thursday: string read FThursday write SetThursday;
    property Friday: string read FFriday write SetFriday;
    property Saturday: string read FSaturday write SetSaturday;
  end;

  TCalendarMonthNames = class(TPersistent)
  private
    FApril: string;
    FNovember: string;
    FMay: string;
    FDecember: string;
    FAugust: string;
    FFebruary: string;
    FJune: string;
    FMarch: string;
    FJuly: string;
    FOctober: string;
    FSeptember: string;
    FJanuary: string;
    FOnChanged: TNotifyEvent;
    procedure SetApril(const Value: string);
    procedure SetAugust(const Value: string);
    procedure SetDecember(const Value: string);
    procedure SetFebruary(const Value: string);
    procedure SetJanuary(const Value: string);
    procedure SetJuly(const Value: string);
    procedure SetJune(const Value: string);
    procedure SetMarch(const Value: string);
    procedure SetMay(const Value: string);
    procedure SetNovember(const Value: string);
    procedure SetOctober(const Value: string);
    procedure SetSeptember(const Value: string);
  protected
    procedure Changed;
  public
    constructor Create; virtual;
    function MonthNameByIndex(AIndex: Integer): string;
    property OnChanged: TNotifyEvent read FOnChanged write FOnChanged;
  published
    property January: string read FJanuary write SetJanuary;
    property February: string read FFebruary write SetFebruary;
    property March: string read FMarch write SetMarch;
    property April: string read FApril write SetApril;
    property May: string read FMay write SetMay;
    property June: string read FJune write SetJune;
    property July: string read FJuly write SetJuly;
    property August: string read FAugust write SetAugust;
    property September: string read FSeptember write SetSeptember;
    property October: string read FOctober write SetOctober;
    property November: string read FNovember write SetNovember;
    property December: string read FDecember write SetDecember;
  end;

  TCalendarSelectDateEvent = procedure(Sender: TObject; ADate: TDateTime) of object;
  TCalendarDates = array of TDateTime;

  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  [JSLibReference('https://download.tmssoftware.com/tmsweb/calendar/TWebCalendar.js', 'https://download.tmssoftware.com/tmsweb/calendar/TWebCalendar.css')]
  {$ENDIF}
  TWebCalendar = class(TWebCustomControl)
  private
    FYear: Integer;
    FOnDateSelected: TCalendarSelectDateEvent;
    FFirstDay: Integer;
    FShowToday: Boolean;
    FMonthNames: TCalendarMonthNames;
    FMultiSelect: Boolean;
    FEnablePastDates: Boolean;
    FDay: Integer;
    FMonth: Integer;
    FNameOfDays: TCalendarWeekDayNames;
    FReadOnly: Boolean;
    FInactiveDays: TCalendarInactiveDays;
    FSelectedDate: TDateTime;
    FHintNext: string;
    FHintPrev: string;
    FMaxDate: TDateTime;
    FMinDate: TDateTime;
    FElementCurrentDateClassName: TElementClassName;
    FElementHeaderClassName: TElementClassName;
    FElementBackgroundClassName: TElementClassName;
    FElementDayNamesClassName: TElementClassName;
    FElementSelectedDateClassName: TElementClassName;
    FOnDateUnselected: TCalendarSelectDateEvent;
    procedure SetSelectedDate(const Value: TDateTime);
    procedure SetYear(const Value: Integer);
    procedure SetMonth(const Value: Integer);
    procedure SetDay(const Value: Integer);
    procedure SetFirstDay(const Value: Integer);
    procedure SetMaxDate(const Value: TDateTime);
    procedure SetMinDate(const Value: TDateTime);
    procedure SetEnablePastDates(const Value: Boolean);
    procedure SetShowToday(const Value: Boolean);
  protected
    procedure Paint; override;
    procedure AppearanceChanged(Sender: TObject);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure EndUpdate; override;
    function SelectedDates: TCalendarDates;
  published
    property Align;
    property Anchors;
    property ChildOrder;
    property DragMode;
    property ShowHint;
    property Visible;
    property TabOrder;
    property TabStop;
    property ElementSelectedDateClassName: TElementClassName read FElementSelectedDateClassName write FElementSelectedDateClassName;
    property ElementCurrentDateClassName: TElementClassName read FElementCurrentDateClassName write FElementCurrentDateClassName;
    property ElementBackgroundClassName: TElementClassName read FElementBackgroundClassName write FElementBackgroundClassName;
    property ElementDayNamesClassName: TElementClassName read FElementDayNamesClassName write FElementDayNamesClassName;
    property ElementHeaderClassName: TElementClassName read FElementHeaderClassName write FElementHeaderClassName;
    property SelectedDate: TDateTime read FSelectedDate write SetSelectedDate;
    property Year: Integer read FYear write SetYear;
    property Font;
    property Month: Integer read FMonth write SetMonth;
    property Day: Integer read FDay write SetDay;
    property HintNext: string read FHintNext write FHintNext;
    property HintPrev: string read FHintPrev write FHintPrev;
    property MaxDate: TDateTime read FMaxDate write SetMaxDate;
    property MinDate: TDateTime read FMinDate write SetMinDate;
    property MultiSelect: Boolean read FMultiSelect write FMultiSelect default True;
    property FirstDay: Integer read FFirstDay write SetFirstDay default 1;
    property EnablePastDates: Boolean read FEnablePastDates write SetEnablePastDates default True;
    property InactiveDays: TCalendarInactiveDays read FInactiveDays write FInactiveDays;
    property NameOfMonths: TCalendarMonthNames read FMonthNames write FMonthNames;
    property NameOfDays: TCalendarWeekDayNames read FNameOfDays write FNameOfDays;
    property ReadOnly: Boolean read FReadOnly write FReadOnly default False;
    property ShowToday: Boolean read FShowToday write SetShowToday default True;
    property OnDateSelected: TCalendarSelectDateEvent read FOnDateSelected write FOnDateSelected;
    property OnDateUnselected: TCalendarSelectDateEvent read FOnDateUnselected write FOnDateUnselected;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnStartDrag;
  end;

implementation

uses
  DateUtils, SysUtils, Math;

{ TCalendar }

procedure TWebCalendar.AppearanceChanged(Sender: TObject);
begin
  Invalidate;
end;

constructor TWebCalendar.Create(AOwner: TComponent);
begin
  inherited;
  Height := 250;
  Width := 250;
  FSelectedDate := Int(Now);
  FYear := YearOf(Now);
  FMonth := MonthOf(Now);
  FDay := DayOf(Now);
  FMultiSelect := True;
  FFirstDay := 1;
  FEnablePastDates := True;
  FReadOnly := False;
  FShowToday := True;
  FHintNext := '';
  FHintPrev := '';
  FElementHeaderClassName := '';
  FElementBackgroundClassName := '';
  FElementDayNamesClassName := '';
  FElementSelectedDateClassName := '';
  FElementCurrentDateClassName := '';
  FInactiveDays := TCalendarInactiveDays.Create;
  FInactiveDays.OnChanged := AppearanceChanged;
  FMonthNames := TCalendarMonthNames.Create;
  FMonthNames.OnChanged := AppearanceChanged;
  FNameOfDays := TCalendarWeekDayNames.Create;
  FNameOfDays.OnChanged := AppearanceChanged;
end;

destructor TWebCalendar.Destroy;
begin
  FInactiveDays.Free;
  FMonthNames.Free;
  FNameOfDays.Free;
  inherited;
end;

procedure TWebCalendar.EndUpdate;
begin
  inherited;
end;

procedure TWebCalendar.Paint;
var
  R, AR: TRect;
  ht: string;
  c, dn, I, row, col, rH: Integer;
  d, mind, maxd: TDateTime;
begin
  inherited;
  ht := FMonthNames.MonthNameByIndex(FMonth) + ' ' + IntToStr(Year);
  R := ClientRect;
  Canvas.Brush.Color := clWhite;
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clWhite;
  Canvas.Pen.Style := psSolid;
  Canvas.Rectangle(R);
  R.Bottom := Round(Height * 0.174242);
  Canvas.Rectangle(R);

  Canvas.Font.Color := clBlack;
  Canvas.Font.Style := [fsBold];
  DrawText(Canvas.Handle, PChar(ht), Length(ht), R, DT_CENTER or DT_SINGLELINE or DT_VCENTER);

  Canvas.Pen.Color := clWindowText;
  Canvas.Pen.Width := 2;
  c := (R.Bottom - R.Top) div 2;
  AR := Rect(R.Left + 8, c - 10, R.Left + 28, c + 10);
  Canvas.MoveTo(AR.Left, AR.Top + (AR.Bottom - AR.Top) div 2);
  Canvas.LineTo(AR.Left + (AR.Right - AR.Left) div 2, AR.Top);
  Canvas.MoveTo(AR.Left, AR.Top + (AR.Bottom - AR.Top) div 2);
  Canvas.LineTo(AR.Left + (AR.Right - AR.Left) div 2, AR.Bottom);
  Canvas.MoveTo(AR.Left, AR.Top + (AR.Bottom - AR.Top) div 2);
  Canvas.LineTo(AR.Right, AR.Top + (AR.Bottom - AR.Top) div 2);

  AR := Rect(R.Right - 8, c - 10, R.Right - 28, c + 10);
  Canvas.MoveTo(AR.Left, AR.Top + (AR.Bottom - AR.Top) div 2);
  Canvas.LineTo(AR.Left + (AR.Right - AR.Left) div 2, AR.Top);
  Canvas.MoveTo(AR.Left, AR.Top + (AR.Bottom - AR.Top) div 2);
  Canvas.LineTo(AR.Left + (AR.Right - AR.Left) div 2, AR.Bottom);
  Canvas.MoveTo(AR.Left, AR.Top + (AR.Bottom - AR.Top) div 2);
  Canvas.LineTo(AR.Right, AR.Top + (AR.Bottom - AR.Top) div 2);

  R.Top := R.Bottom;
  R.Bottom := R.Top + Round(Height * 0.1628787);
  c := Round(R.Right * 0.1428);
  AR := Rect(R.Left, R.Top, c, R.Bottom);
  Canvas.Brush.Color := clHighlight;
  Canvas.Pen.Color := clHighlight;
  Canvas.Pen.Width := 1;
  Canvas.Font.Color := clWhite;
  Canvas.Font.Style := [fsBold];

  dn := FirstDay;
  for I := 0 to 6 do
  begin
    Canvas.Rectangle(AR);
    ht := FNameOfDays.DayNameByIndex(dn);
    DrawText(Canvas.Handle, PChar(ht), Length(ht), AR, DT_CENTER or DT_SINGLELINE or DT_VCENTER);
    dn := (dn + 1) mod 7;
    if dn = 0 then
      dn := 7;
    AR.Left := AR.Right;
    AR.Right := AR.Left + c;
  end;

  row := 1;
  col := DayOfWeek(EncodeDate(FYear, FMonth, 1)) - FirstDay;
  if col < 0 then
    col := col + 7;

  dn := DaysInMonth(EncodeDate(FYear, FMonth, 1));
  rH := Round(Height * 0.6628787 / Ceil((dn + col) / 7));

  R.Top := R.Bottom;
  R.Bottom := R.Top + rH;
  R.Right := R.Left + c;

  for I := 1 to dn do
  begin
    d := EncodeDate(FYear, FMonth, I);
    mind := MinDate;
    maxd := MaxDate;

    if MinDate = 0 then
      mind := d;

    if MaxDate = 0 then
      maxd := d;

    Canvas.Font.Style := [];
    if CompareDate(d, FSelectedDate) = 0 then
    begin
      Canvas.Brush.Color := clHighlight;
      Canvas.Pen.Color := clHighlight;
      Canvas.Font.Color := clWhite;
    end
    else if (Int(d) < Int(mind)) or (Int(d) > Int(maxd)) or (not FEnablePastDates and (CompareDate(d, Int(Now)) < 0)) or FInactiveDays.DateEnabledByIndex(col + 1) then
    begin
      Canvas.Brush.Color := clWhite;
      Canvas.Pen.Color := clWhite;
      Canvas.Font.Color := clSilver;
    end
    else if CompareDate(d, Int(Now)) = 0 then
    begin
      Canvas.Brush.Color := clSilver;
      Canvas.Pen.Color := clSilver;
      Canvas.Font.Color := clBlack;
    end
    else
    begin
      Canvas.Brush.Color := clWhite;
      Canvas.Pen.Color := clWhite;
      Canvas.Font.Color := clBlack;
    end;

    AR := Rect(R.Left + col * c, R.Top + (row - 1) * rH, R.Right + col * c, R.Bottom + (row - 1) * rH);
    Canvas.Rectangle(AR);
    DrawText(Canvas.Handle, PChar(IntToStr(I)), Length(IntToStr(I)), AR, DT_CENTER or DT_SINGLELINE or DT_VCENTER);
    Inc(col);
    if col > 6 then
    begin
      col := 0;
      Inc(row);
    end;
  end;
end;

function TWebCalendar.SelectedDates: TCalendarDates;
begin
  SetLength(Result, 0);
end;

procedure TWebCalendar.SetDay(const Value: Integer);
begin
  FDay := Value;
  SelectedDate := EncodeDate(FYear, FMonth, Value);
end;

procedure TWebCalendar.SetEnablePastDates(const Value: Boolean);
begin
  FEnablePastDates := Value;
  Invalidate;
end;

procedure TWebCalendar.SetFirstDay(const Value: Integer);
begin
  FFirstDay := Value;
  Invalidate;
end;

procedure TWebCalendar.SetMaxDate(const Value: TDateTime);
begin
  FMaxDate := Value;
  Invalidate;
end;

procedure TWebCalendar.SetMinDate(const Value: TDateTime);
begin
  FMinDate := Value;
  Invalidate;
end;

procedure TWebCalendar.SetMonth(const Value: Integer);
begin
  FMonth := Value;
  SelectedDate := EncodeDate(FYear, Value, FDay);
end;

procedure TWebCalendar.SetSelectedDate(const Value: TDateTime);
begin
  FSelectedDate := Value;
  FYear := YearOf(Value);
  FMonth := MonthOf(Value);
  FDay := DayOf(Value);
  Invalidate;
end;

procedure TWebCalendar.SetShowToday(const Value: Boolean);
begin
  FShowToday := Value;
  Invalidate;
end;

procedure TWebCalendar.SetYear(const Value: Integer);
begin
  FYear := Value;
  SelectedDate := EncodeDate(Value, FMonth, FDay);
end;

{ TCalendarInactiveDays }

procedure TCalendarInactiveDays.Changed;
begin
  if Assigned(OnChanged) then
    OnChanged(Self);
end;

constructor TCalendarInactiveDays.Create;
begin
  FSunday := False;
  FMonday := False;
  FTuesday := False;
  FWednesday := False;
  FThursday := False;
  FFriday := False;
  FSaturday := False;
end;

function TCalendarInactiveDays.DateEnabledByIndex(AIndex: Integer): Boolean;
begin
  case AIndex of
    1: Result := FSunday;
    2: Result := FMonday;
    3: Result := FTuesday;
    4: Result := FWednesday;
    5: Result := FThursday;
    6: Result := FFriday;
    7: Result := FSaturday;
    else Result := False;
  end;
end;

procedure TCalendarInactiveDays.SetFriday(const Value: Boolean);
begin
  FFriday := Value;
  Changed;
end;

procedure TCalendarInactiveDays.SetMonday(const Value: Boolean);
begin
  FMonday := Value;
  Changed;
end;

procedure TCalendarInactiveDays.SetSaturday(const Value: Boolean);
begin
  FSaturday := Value;
  Changed;
end;

procedure TCalendarInactiveDays.SetSunday(const Value: Boolean);
begin
  FSunday := Value;
  Changed;
end;

procedure TCalendarInactiveDays.SetThursday(const Value: Boolean);
begin
  FThursday := Value;
  Changed;
end;

procedure TCalendarInactiveDays.SetTuesday(const Value: Boolean);
begin
  FTuesday := Value;
  Changed;
end;

procedure TCalendarInactiveDays.SetWednesday(const Value: Boolean);
begin
  FWednesday := Value;
  Changed;
end;

{ TCalendarWeekDayNames }

procedure TCalendarWeekDayNames.Changed;
begin
  if Assigned(OnChanged) then
    OnChanged(Self);
end;

constructor TCalendarWeekDayNames.Create;
begin
  FSunday := 'Sun';
  FMonday := 'Mon';
  FTuesday := 'Tue';
  FWednesday := 'Wed';
  FThursday := 'Thu';
  FFriday := 'Fri';
  FSaturday := 'Sat';
end;

function TCalendarWeekDayNames.DayNameByIndex(AIndex: Integer): string;
begin
  case AIndex of
    1: Result := FSunday;
    2: Result := FMonday;
    3: Result := FTuesday;
    4: Result := FWednesday;
    5: Result := FThursday;
    6: Result := FFriday;
    7: Result := FSaturday;
    else Result := '';
  end;
end;

procedure TCalendarWeekDayNames.SetFriday(const Value: string);
begin
  FFriday := Value;
  Changed;
end;

procedure TCalendarWeekDayNames.SetMonday(const Value: string);
begin
  FMonday := Value;
  Changed;
end;

procedure TCalendarWeekDayNames.SetSaturday(const Value: string);
begin
  FSaturday := Value;
  Changed;
end;

procedure TCalendarWeekDayNames.SetSunday(const Value: string);
begin
  FSunday := Value;
  Changed;
end;

procedure TCalendarWeekDayNames.SetThursday(const Value: string);
begin
  FThursday := Value;
  Changed;
end;

procedure TCalendarWeekDayNames.SetTuesday(const Value: string);
begin
  FTuesday := Value;
  Changed;
end;

procedure TCalendarWeekDayNames.SetWednesday(const Value: string);
begin
  FWednesday := Value;
  Changed;
end;

{ TCalendarMonthNames }

procedure TCalendarMonthNames.Changed;
begin
  if Assigned(OnChanged) then
    OnChanged(Self);
end;

constructor TCalendarMonthNames.Create;
begin
  FJanuary := 'January';
  FFebruary := 'February';
  FMarch := 'March';
  FApril := 'April';
  FMay := 'May';
  FJune := 'June';
  FJuly := 'July';
  FAugust := 'August';
  FSeptember := 'September';
  FOctober := 'October';
  FNovember := 'November';
  FDecember := 'December';
end;

function TCalendarMonthNames.MonthNameByIndex(AIndex: Integer): string;
begin
  case AIndex of
    1: Result := FJanuary;
    2: Result := FFebruary;
    3: Result := FMarch;
    4: Result := FApril;
    5: Result := FMay;
    6: Result := FJune;
    7: Result := FJuly;
    8: Result := FAugust;
    9: Result := FSeptember;
    10: Result := FOctober;
    11: Result := FNovember;
    12: Result := FDecember;
    else Result := '';
  end;
end;

procedure TCalendarMonthNames.SetApril(const Value: string);
begin
  FApril := Value;
  Changed;
end;

procedure TCalendarMonthNames.SetAugust(const Value: string);
begin
  FAugust := Value;
  Changed;
end;

procedure TCalendarMonthNames.SetDecember(const Value: string);
begin
  FDecember := Value;
  Changed;
end;

procedure TCalendarMonthNames.SetFebruary(const Value: string);
begin
  FFebruary := Value;
  Changed;
end;

procedure TCalendarMonthNames.SetJanuary(const Value: string);
begin
  FJanuary := Value;
  Changed;
end;

procedure TCalendarMonthNames.SetJuly(const Value: string);
begin
  FJuly := Value;
  Changed;
end;

procedure TCalendarMonthNames.SetJune(const Value: string);
begin
  FJune := Value;
  Changed;
end;

procedure TCalendarMonthNames.SetMarch(const Value: string);
begin
  FMarch := Value;
  Changed;
end;

procedure TCalendarMonthNames.SetMay(const Value: string);
begin
  FMay := Value;
  Changed;
end;

procedure TCalendarMonthNames.SetNovember(const Value: string);
begin
  FNovember := Value;
  Changed;
end;

procedure TCalendarMonthNames.SetOctober(const Value: string);
begin
  FOctober := Value;
  Changed;
end;

procedure TCalendarMonthNames.SetSeptember(const Value: string);
begin
  FSeptember := Value;
  Changed;
end;

end.
