{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018 - 2019                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

unit WEBLib.FlexControls;

interface

uses
  {$IFNDEF LCLTMSWEB}
  System.Classes, Vcl.Graphics, Vcl.Grids, Vcl.Controls, WEBLib.Controls, System.Types, Web;
  {$ENDIF}
  {$IFDEF LCLTMSWEB}
  Classes, Graphics, Grids, Controls, WEBLib.Controls, Types, Web;
  {$ENDIF}

type
  {$IFNDEF LCLTMSWEB}
  [ComponentPlatforms(TMSWebPlatform)]
  {$ENDIF}
  TWebStretchPanel = class(TWebCustomControl)
  private
    FFixedHeight: integer;
    procedure SetFixedHeight(const Value: integer);
  protected
    procedure UpdateChildAnchors;
    procedure Paint; override;
  public
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    constructor Create(Aowner: TComponent); override;
  published
    property Align;
    {$IFNDEF LCLTMSWEB}
    property AlignWithMargins;
    {$ENDIF}
    property Anchors;
    property BorderColor;
    property BorderStyle;
    property ChildOrder;
    property Color;
    property FixedHeight: integer read FFixedHeight write SetFixedHeight default 100;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;


implementation

{ TWebStretchPanel }

constructor TWebStretchPanel.Create(Aowner: TComponent);
begin
  inherited;
  FFixedHeight := 100;
  ControlStyle := ControlStyle + [csAcceptsControls];
  Width := 320;
  Height := 250;
end;

procedure TWebStretchPanel.Paint;
begin
  inherited;
  Canvas.Pen.Color := clGray;
  Canvas.Pen.Style := psDot;
  Canvas.Pen.Width := 1;

  Canvas.MoveTo(0, Height - FixedHeight);
  Canvas.LineTo(Width, Height - FixedHeight);
end;

procedure TWebStretchPanel.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  UpdateChildAnchors;
  inherited;

end;

procedure TWebStretchPanel.SetFixedHeight(const Value: integer);
begin
  if (FFixedHeight <> Value) then
  begin
    FFixedHeight := Value;
    Invalidate;
  end;
end;

procedure TWebStretchPanel.UpdateChildAnchors;
var
  i: integer;
begin
  for i := 0 to ControlCount - 1 do
  begin
    if Controls[i].Top > Height - FixedHeight then
      Controls[i].Anchors := Controls[i].Anchors - [akTop] + [akBottom]
    else
      Controls[i].Anchors := Controls[i].Anchors + [akTop] - [akBottom];
  end;

end;

end.
