{********************************************************************}
{                                                                    }
{ written by TMS Software                                            }
{            copyright (c) 2018 - 2020                               }
{            Email : info@tmssoftware.com                            }
{            Web : http://www.tmssoftware.com                        }
{                                                                    }
{ The source code is given as is. The author is not responsible      }
{ for any possible damage done due to the use of this code.          }
{ The complete source code remains property of the author and may    }
{ not be distributed, published, given or sold in any form as such.  }
{ No parts of the source code can be included in any other component }
{ or application without written authorization of the author.        }
{********************************************************************}

{ IDE options frame for webcore options }
unit FraWebCoreOptions;

{$mode objfpc}{$H+}
{$Inline on}

interface

uses
  Classes, SysUtils, Math, RegExpr,
  // LCL
  Forms, StdCtrls, Dialogs, Controls,
  // LazUtils
  LazFileCache, LazFileUtils,
  // IdeIntf
  IDEOptionsIntf, IDEOptEditorIntf, IDEUtils,
  // WebCore
  TMSWebCoreIDEOptions;

Type
  { TWebCoreOptionsFrame }

  TWebCoreOptionsFrame = class(TAbstractIDEOptionsEditor)
    BTMSWebDir: TButton;
    BUserJavascriptLibManager: TButton;
    BJavascriptLibConfig: TButton;
    WebCoreShowValuesButton: TButton;
    CBComponentUnits: TComboBox;
    CBJavaScriptLibConfig: TComboBox;
    CBBrowser: TComboBox;
    CBTMSWebDir: TComboBox;
    CBUserJavaScriptLibConfig: TComboBox;
    CBURL: TComboBox;
    CBWebserverParams: TComboBox;
    CBWebServerPort: TComboBox;
    LCBWebServerPort: TLabel;
    LComponentUnits: TLabel;
    LUserJSLibConfig: TLabel;
    Browser: TLabel;
    LTMSWebDir: TLabel;
    LJSFileGeneration: TLabel;
    LCBWebserverParams: TLabel;
    LCBURL: TLabel;
    BWebRunner: TButton;
    CBSingleJSFile: TCheckBox;
    CBWebRunner: TComboBox;
    CBWebserverVisibility: TComboBox;
    LCBJavaScriptLibConfig: TLabel;
    LCBWebserverVisibility: TLabel;
    BWebServer: TButton;
    CBWebServer: TComboBox;
    LCBWebServer: TLabel;
    LWebRunner: TLabel;
    BWebCompiler: TButton;
    CBWebCompiler: TComboBox;
    LCBWebCompiler: TLabel;
    WebCoreResetButton: TButton;
    procedure BTMSWebDirClick(Sender: TObject);
    procedure BUserJavascriptLibManagerClick(Sender: TObject);
    procedure BJavascriptLibConfigClick(Sender: TObject);
    procedure BWebRunnerClick(Sender: TObject);
    procedure BWebServerClick(Sender: TObject);
    procedure BWebCompilerClick(Sender: TObject);
    procedure WebCoreResetButtonClick(Sender: TObject);
    procedure WebCoreShowValuesButtonClick(Sender: TObject);
  private
    FOldTMSWebDir, FOldWebCompiler, FOldWebServer, FOldWebServerParams,
      FOldJavaScriptLibManager, FOldUserJavaScriptLibManager, FOldURL,
      FOldWebRunner, FOldComponentUnits: String;
    FOldBrowser: TBrowser;
    FOldWebServerVisibility: TWebServerVisibility;
    FOldSingleJSFile: Boolean;
  protected
    function SelectDir(CB: TComboBox; ATitle: String): String; virtual;
    function SelectExe(CB: TComboBox; ATitle: String): String; virtual;
    function SelectIni(CB: TComboBox; ATitle: String): String; virtual;
    procedure DoReadSettings; virtual;
    procedure DoWriteSettings; virtual;
    procedure CheckVersionFile(WebDir: string; Msg: TStrings); virtual;
  public
    function Check: Boolean; override;
    function GetTitle: String; override;
    procedure Setup({%H-}ADialog: TAbstractOptionsEditorDialog); override;
    procedure ReadSettings({%H-}AOptions: TAbstractIDEOptions); override;
    procedure WriteSettings({%H-}AOptions: TAbstractIDEOptions); override;
    procedure RestoreSettings({%H-}AOptions: TAbstractIDEOptions); override;
    class function SupportedOptionsClass: TAbstractIDEOptionsClass; override;
  end;

implementation

{$R *.lfm}

{ TWebCoreOptionsFrame }

procedure TWebCoreOptionsFrame.BWebCompilerClick(Sender: TObject);
var
  AFilename: String;
begin
  aFileName:=SelectExe(CBWebCompiler,'Select TMS Web Compiler executable');
  if (aFileName<>'') then
    WebCoreOptions.WebCompiler:=aFileName;
end;

procedure TWebCoreOptionsFrame.WebCoreResetButtonClick(Sender: TObject);
begin
  WebCoreOptions.Reset;
  DoReadSettings;
end;

procedure TWebCoreOptionsFrame.WebCoreShowValuesButtonClick(Sender: TObject);
var
  Frm: TForm;
  Memo1: TMemo;
  sl: TStringList;
  URL, BrowserPath, WebDir, WebCompiler, WebServerPath,
    WebRunner: String;
  BrowserParams: TStringList;
begin
  sl:=TStringList.Create;
  BrowserParams:=TStringList.Create;
  Frm:=TForm.Create(nil);
  try
    DoWriteSettings;
    // webdir
    WebDir:=WebCoreOptions.GetParsedOptionValue(wccoTMSWebDir);
    sl.Add('WebDir='+WebDir);
    if not DirectoryExistsUTF8(WebDir) then
      sl.Add('Warning: missing WebDir')
    else
      CheckVersionFile(WebDir,sl);

    // webcompiler
    WebCompiler:=WebCoreOptions.GetParsedOptionValue(wccoWebCompiler);
    sl.Add('WebCompiler='+WebCompiler);
    if not FileExistsUTF8(WebCompiler) then
      sl.Add('Warning: missing WebCompiler');
    // webserver path
    WebServerPath:=WebCoreOptions.GetParsedOptionValue(wccoWebServer);
    sl.Add('WebServerPath='+WebServerPath);
    if (WebServerPath<>'') and not FileExistsUTF8(WebServerPath) then
      sl.Add('Warning: missing WebServerPath');
    // webserver port
    sl.Add('WebServerPort='+WebCoreOptions.GetParsedOptionValue(wccoWebServerPort));
    // webserver params
    sl.Add('WebServerParams='+WebCoreOptions.GetParsedOptionValue(wccoWebServerParams));
    // runner
    WebRunner:=WebCoreOptions.GetParsedOptionValue(wccoWebRunner);
    sl.Add('WebRunner='+WebRunner);
    if (WebRunner<>'') and not FileExistsUTF8(WebRunner) then
      sl.Add('Warning: missing WebRunner');
    // javascript lib config
    sl.Add('JavascriptLibConfig='+WebCoreOptions.GetParsedOptionValue(wccoJavascriptLibManager));
    // javascript user lib config
    sl.Add('JavascriptUserLibConfig='+WebCoreOptions.GetParsedOptionValue(wccoUserJavascriptLibManager));
    // URL
    URL:=WebCoreOptions.GetParsedOptionValue(wccoURL);
    sl.Add('WebServerURL='+URL);
    // component units
    sl.Add('ComponentUnits='+WebCoreOptions.GetParsedOptionValue(wccoComponentUnits));
    // browser
    BrowserPath:=WebCoreOptions.GetBrowserPath(URL,BrowserParams);
    sl.Add('Browser='+BrowserPath);
    Memo1:=TMemo.Create(Frm);
    Memo1.Lines.Assign(sl);
    Memo1.Align:=alClient;
    Memo1.Parent:=Frm;
    Memo1.WordWrap:=true;;
    Memo1.ScrollBars:=ssAutoVertical;
    Frm.Position:=poScreenCenter;
    Frm.ShowModal;
  finally
    Frm.Free;
    sl.Free;
    BrowserParams.Free;
  end;
end;

procedure TWebCoreOptionsFrame.BWebServerClick(Sender: TObject);

var
  AFilename: String;

begin
  aFileName:=SelectExe(CBWebCompiler,'Select webserver executable');
  if (aFileName<>'') then
    WebCoreOptions.WebServer:=aFileName;
end;

function TWebCoreOptionsFrame.SelectIni(CB: TComboBox; ATitle: String): String;
var
  OpenDialog: TOpenDialog;
begin
  Result:='';
  OpenDialog:=TOpenDialog.Create(nil);
  try
    //InputHistories.ApplyFileDialogSettings(OpenDialog);
    OpenDialog.Options:=OpenDialog.Options+[ofPathMustExist];
    OpenDialog.Title:=ATitle;
    OpenDialog.Filter:='Configuration files|*.ini';
    if OpenDialog.Execute then begin
      Result:=CleanAndExpandFilename(OpenDialog.Filename);
      SetComboBoxText(CB,Result,cstFilename,30);
    end;
  finally
    OpenDialog.Free;
  end;
end;

procedure TWebCoreOptionsFrame.DoReadSettings;
begin
  With WebCoreOptions do
    begin
    SetComboBoxText(CBTMSWebDir,ChompPathDelim(TMSWebDir),cstFilename,30);
    SetComboBoxText(CBWebCompiler,WebCompiler,cstFilename,30);
    SetComboBoxText(CBWebServer,WebServer,cstFilename,30);
    SetComboBoxText(CBWebserverParams,WebServerParams,cstCaseInsensitive,30);
    SetComboBoxText(CBWebServerPort,WebServerPort,cstCaseInsensitive,30);
    SetComboBoxText(CBWebRunner,WebRunner,cstFilename,30);
    SetComboBoxText(CBJavaScriptLibConfig,JavaScriptLibConfig,cstFilename,30);
    SetComboBoxText(CBUserJavaScriptLibConfig,UserJavaScriptLibConfig,cstFilename,30);
    SetComboBoxText(CBURL,WebCoreURL,cstFilename,30);
    SetComboBoxText(CBComponentUnits,ComponentUnits,cstCaseInsensitive,30);
    SetComboBoxText(CBBrowser,BrowserName[Browser],cstCaseInsensitive,5);
    CBWebServerVisibility.Itemindex:=Ord(WebServerVisibility);
    CBSingleJSFile.Checked:=SingleJSFile;
    end;
end;

procedure TWebCoreOptionsFrame.DoWriteSettings;
begin
  With WebCoreOptions do
  begin
    TMSWebDir:=TrimFilename(CBTMSWebDir.Text);
    WebCompiler:=TrimFilename(CBWebCompiler.Text);
    WebServer:=TrimFilename(CBWebServer.Text);
    WebServerParams:=CBWebserverParams.Text;
    WebServerPort:=CBWebServerPort.Text;
    WebRunner:=TrimFilename(CBWebRunner.Text);
    JavaScriptLibConfig:=TrimFilename(CBJavaScriptLibConfig.Text);
    UserJavaScriptLibConfig:=TrimFilename(CBUserJavaScriptLibConfig.Text);
    WebCoreURL:=Trim(CBURL.Text);
    ComponentUnits:=Trim(CBComponentUnits.Text);
    Browser:=NameToBrowser(CBBrowser.Text);
    WebServerVisibility:=TWebServerVisibility(Max(0,CBWebServerVisibility.ItemIndex));
    SingleJSFile:=CBSingleJSFile.Checked;
  end;
end;

procedure TWebCoreOptionsFrame.CheckVersionFile(WebDir: string; Msg: TStrings);
const
  ProductVerName = 'TMSProductInfo_ProductVersion';
var
  InfoFilename, Line, VerStr: String;
  Lines: TStringList;
  i: Integer;
  r: TRegExpr;
begin
  InfoFilename:=AppendPathDelim(WebDir)+'TMSProductInfo.pas';
  if not FileExistsUTF8(InfoFilename) then
  begin
    Msg.Add('Warning: missing version file ['+InfoFilename+']');
    exit;
  end;
  r := TRegExpr.Create;
  Lines:=TStringList.Create;
  try
    try
      r.Expression := ProductVerName+' *= *''([0-9.]+)''';
      Lines.LoadFromFile(InfoFilename);
      VerStr:='';
      for i:=0 to Lines.Count-1 do
      begin
        Line:=Lines[i];
        if not r.Exec(Line) then
          continue;
        VerStr:=r.Match[1];
        break;
      end;
      if VerStr='' then
        Msg.Add('Warning: InfoFilename=['+InfoFilename+']: missing const '+ProductVerName)
      else
        Msg.Add(ProductVerName+'='+VerStr);
    except
      on E: Exception do begin
        Msg.Add('Warning: InfoFilename=['+InfoFilename+'] E='+E.Message);
      end;
    end;
  finally
    r.Free;
    Lines.Free;
  end;
end;

function TWebCoreOptionsFrame.Check: Boolean;
var
  NewTMSWebDir: TCaption;
begin
  Result:=inherited Check;
  With WebCoreOptions do
    begin
    NewTMSWebDir:=ChompPathDelim(TrimFilename(CBTMSWebDir.Text));
    if TMSWebDir<>NewTMSWebDir then
      if not WebCoreOptions.CheckTMSWebDir(NewTMSWebDir) then exit(false);
    // ToDo: webrunner, ...
    end;
  Result:=true;
end;

function TWebCoreOptionsFrame.SelectExe(CB: TComboBox; ATitle: String): String;
var
  OpenDialog: TOpenDialog;
begin
  Result:='';
  OpenDialog:=TOpenDialog.Create(nil);
  try
    //InputHistories.ApplyFileDialogSettings(OpenDialog);
    OpenDialog.Options:=OpenDialog.Options+[ofPathMustExist];
    OpenDialog.Title:=ATitle;
{$IFDEF WINDOWS}
    OpenDialog.Filter:='Executables|*.exe';
{$ENDIF WINDOWS}
    if OpenDialog.Execute then begin
      Result:=CleanAndExpandFilename(OpenDialog.Filename);
      SetComboBoxText(CB,Result,cstFilename,30);
    end;
  finally
    OpenDialog.Free;
  end;
end;

function TWebCoreOptionsFrame.SelectDir(CB: TComboBox; ATitle: String): String;
var
  OpenDialog: TSelectDirectoryDialog;
begin
  Result:='';
  OpenDialog:=TSelectDirectoryDialog.Create(nil);
  try
    //InputHistories.ApplyFileDialogSettings(OpenDialog);
    OpenDialog.Options:=OpenDialog.Options+[ofPathMustExist];
    OpenDialog.Title:=aTitle;
    if OpenDialog.Execute then begin
      Result:=CleanAndExpandFilename(OpenDialog.Filename);
      SetComboBoxText(CB,Result,cstFilename,30);
    end;
  finally
    OpenDialog.Free;
  end;
end;

procedure TWebCoreOptionsFrame.BWebRunnerClick(Sender: TObject);
var
  AFilename: String;
begin
  aFileName:=SelectExe(CBWebRunner,'Select web runner executable');
  if (AFileName<>'') then
    WebCoreOptions.WebRunner:=AFileName;
end;

procedure TWebCoreOptionsFrame.BJavascriptLibConfigClick(Sender: TObject);
var
  AFilename: String;
begin
  aFileName:=SelectIni(CBJavaScriptLibConfig,'Select JavaScript manager library config file');
  if (AFileName<>'') then
    WebCoreOptions.JavaScriptLibConfig:=AFilename;
end;

procedure TWebCoreOptionsFrame.BUserJavascriptLibManagerClick(Sender: TObject);
begin
  SelectIni(CBUserJavaScriptLibConfig,'Select user JavaScript manager library config file');
end;

procedure TWebCoreOptionsFrame.BTMSWebDirClick(Sender: TObject);
var
  OldDir: TCaption;
begin
  OldDir:=CBTMSWebDir.Text;
  SelectDir(CBTMSWebDir,'Select directory TMS Web Core Lazarus');
  if not WebCoreOptions.CheckTMSWebDir(CBTMSWebDir.Text) then
    CBTMSWebDir.Text:=OldDir;
end;

function TWebCoreOptionsFrame.GetTitle: String;
begin
  Result:='TMS Web Core';
end;

procedure TWebCoreOptionsFrame.Setup(ADialog: TAbstractOptionsEditorDialog);
var
  WebRunnerName,
  WebCompilerName,
  ServerName : String;
  sl: TStringList;
  b: TBrowser;
begin
  WebCompilerName:=ExtractFileName(GetStandardCachedOption(wccoWebCompiler));
  ServerName:=ExtractFileName(GetStandardCachedOption(wccoWebServer));
  WebRunnerName:=ExtractFileName(GetStandardCachedOption(wccoWebRunner));
  CBTMSWebDir.Hint:='You can use IDE macros like $Env().';
  CBWebCompiler.Hint:='You can use IDE macros like $MakeExe(). Without a full path, '+WebCompilerName+' is searched in PATH.';
  CBWebServer.Hint:='You can use IDE macros like $MakeExe(). Without a full path, '+ServerName+' is searched in PATH.';
  CBWebRunner.Hint:='You can use IDE macros like $MakeExe(). Without a full path, '+WebRunnerName+' is searched in PATH.';
  BTMSWebDir.Hint:='Browse';
  BWebCompiler.Hint:='Browse';
  BWebServer.Hint:='Browse';
  BWebRunner.Hint:='Browse';

  sl:=TStringList.Create;
  for b:=bDefault to high(TBrowser) do
    sl.Add(BrowserName[b]);
  CBBrowser.Items.Assign(sl);
  sl.Free;
end;

procedure TWebCoreOptionsFrame.ReadSettings(AOptions: TAbstractIDEOptions);
begin
  With WebCoreOptions do
    begin
    FOldTMSWebDir:=TMSWebDir;
    FOldWebCompiler:=WebCompiler;
    FOldWebServer:=WebServer;
    FOldWebServerParams:=WebServerParams;
    FOldWebRunner:=WebRunner;
    FOldJavaScriptLibManager:=JavaScriptLibConfig;
    FOldUserJavaScriptLibManager:=UserJavaScriptLibConfig;
    FOldURL:=WebCoreURL;
    FOldComponentUnits:=ComponentUnits;
    FOldBrowser:=Browser;
    FOldWebServerVisibility:=WebServerVisibility;
    FOldSingleJSFile:=SingleJSFile;
    end;
  DoReadSettings;
end;

procedure TWebCoreOptionsFrame.WriteSettings(AOptions: TAbstractIDEOptions);
var
  NewTMSWebDir: TCaption;
  TMSWebDirChanged: Boolean;
begin
  NewTMSWebDir:=TrimFilename(CBTMSWebDir.Text);
  TMSWebDirChanged:=WebCoreOptions.TMSWebDir<>NewTMSWebDir;
  DoWriteSettings;
  If WebCoreOptions.Modified then
    WebCoreOptions.Save;
  if TMSWebDirChanged then
    WebCoreOptions.FixTMSWebDir;
end;

procedure TWebCoreOptionsFrame.RestoreSettings(AOptions: TAbstractIDEOptions);
begin
  inherited RestoreSettings(AOptions);
  With WebCoreOptions do
    begin
    TMSWebDir:=FOldTMSWebDir;
    WebCompiler:=FOldWebCompiler;
    WebServer:=FOldWebServer;
    WebServerParams:=FOldWebServerParams;
    WebRunner:=FOldWebRunner;
    JavaScriptLibConfig:=FOldJavaScriptLibManager;
    UserJavaScriptLibConfig:=FOldUserJavaScriptLibManager;
    WebCoreURL:=FOldURL;
    ComponentUnits:=FOldComponentUnits;
    Browser:=FOldBrowser;
    WebServerVisibility:=FOldWebServerVisibility;
    SingleJSFile:=FOldSingleJSFile;
    end;
end;

class function TWebCoreOptionsFrame.SupportedOptionsClass: TAbstractIDEOptionsClass;
begin
  Result:=IDEEditorGroups.GetByIndex(GroupEnvironment)^.GroupClass;
end;

end.

